import QtQuick 2.0
import QtQuick.XmlListModel 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1

import "../databasemodule_v2.js" as DB

Page {
    id: appendFeedPage

    title: "Append feed"
    flickable: null
    visible: false

    property bool isDirty: false // Need to clear all page content.
    property int selectedCount: 0
    property var selectedFeeds: null


    tools: ToolbarItems {
        id: appendFeedTools

        opened: true
        locked: true

        back: Button {
            anchors.verticalCenter: parent.verticalCenter
            action: Action {
                text:  i18n.tr("Cancel")

                onTriggered:
                {
                    pageStack.pop()
                }
            }
        }

        ActivityIndicator {
            id: checkRunning

            visible: googleFeedApi.inProgress
            running: googleFeedApi.inProgress

            anchors.verticalCenter: parent.verticalCenter
        }

        Button {
            id: nextBtn
            anchors.verticalCenter: parent.verticalCenter

            enabled: !googleFeedApi.inProgress && selectedCount > 0
            action: Action {
                text:  i18n.tr("Next")

                onTriggered:
                {
                    if (!nextBtn.enabled)
                        return

                    selectedFeeds = []
                    for (var i = 0; i < searchResultsModel.count; i++) {
                        var curItem = searchResultsModel.get(i)

                        if (curItem.isSelected) {
                            selectedFeeds.push(curItem)
                        }
                    }

                    pageStack.push(chooseTopicPage, {"feedsToAdd" : selectedFeeds})
                }
            }
        }
    }

    function reloadPageContent() {
        if (isDirty) {
            // isUrlEntered = false
            tfFeedUrl.text = ""
            clearModelDependData()
        }
        isDirty = false
    }

    /* Clear model and model's depend data.
     * Currently only selectedCount.
     */
    function clearModelDependData() {
        searchResultsModel.clear()
        selectedCount = 0
    }

    // -------------------------------------       GOOGLE API

    Connections {
        target: googleFeedApi

        onFindResult: {

            if (result.responseStatus !== 200) {
                // Handle it
                console.log(JSON.stringify(result))
                return
            }

            clearModelDependData()

            var entries = result.responseData.entries

            for (var i = 0; i < entries.length; i++) {
                searchResultsModel.append({"url" : entries[i].url,
                                                 "title" : entries[i].title,
                                                 "contentSnippet" : entries[i].contentSnippet,
                                                 "url" : entries[i].url,
                                                 "link" : entries[i].link,
                                                 "isSelected" : false})
            }
        }

        onLoadResult: {
            if (result.responseStatus !== 200) {
                // Handle it
                console.log(JSON.stringify(result))
                return
            }

            clearModelDependData()

            var feed = result.responseData.feed

            searchResultsModel.append({"url" : feed.feedUrl,
                                             "title" : feed.title,
                                             "contentSnippet" : feed.contentSnippet,
                                             "url" : feed.url,
                                             "link" : feed.link,
                                             "isSelected" : false})
        }
    }

    QtObject {
        id: googleFeedApi

        property var __doc: null
        property bool inProgress: __doc != null

        signal findResult(var result)
        signal loadResult(var result)

        /* Find feed by keywords.
         */
        function findFeeds(searchKeywords) {
            if (__doc != null) {
                __doc.abort()
                //__doc = null // ?
            }

            __doc = new XMLHttpRequest()
            var doc = __doc

            doc.onreadystatechange = function() {

                if (doc.readyState === XMLHttpRequest.DONE) {

                    var resObj
                    if (doc.status == 200) {
                        resObj = JSON.parse(doc.responseText)
                    } else { // Error
                        resObj = {"responseDetails" : doc.statusText,
                            "responseStatus" : doc.status}
                    }

                    __doc = null
                    findResult(resObj)
                }
            }

            var searchKeywordsEncoded = encodeURIComponent(searchKeywords)

            var baseUrl = "https://ajax.googleapis.com/ajax/services/feed/find?v=1.0&q="
            var finalRequest = baseUrl + searchKeywordsEncoded

            doc.open("GET", finalRequest, true);
            doc.send();
        }

        /* Load feed by URL.
         */
        function loadFeed(feedUrl) {
            if (__doc != null) {
                __doc.abort()
                //__doc = null // ?
            }

            __doc = new XMLHttpRequest()
            var doc = __doc

            doc.onreadystatechange = function() {

                if (doc.readyState === XMLHttpRequest.DONE) {

                    var resObj
                    if (doc.status == 200) {
                        resObj = JSON.parse(doc.responseText)
                    } else { // Error
                        resObj = {"responseDetails" : doc.statusText,
                            "responseStatus" : doc.status}
                    }

                    __doc = null
                    loadResult(resObj)
                }
            }

            var encodedUrl = encodeURIComponent(feedUrl)
            var baseUrl = "https://ajax.googleapis.com/ajax/services/feed/load?v=1.0&q="
            var finalRequest = baseUrl + encodedUrl

            /* Add some optional params.
             * May be usable:
             * hl       - host language, for example "hl=ru", default en.
             * num      - number of entries, for example "num=50", default 4, maximum 100.
             * output   - format of output, for example "output=json", may be xml, json_xml, json.
             */
            doc.open("GET", finalRequest, true);
            doc.send();
        }
    } // QtObject googleFeedApi

    // -------------------------------------       GOOGLE API

    Column {
        id: appendFeedColumn

        anchors.top: parent.top
        anchors.topMargin: units.gu(2)
        width: parent.width
        spacing: units.gu(2)

        TextField {
            id: tfFeedUrl

            placeholderText: i18n.tr("Enter keywords or direct url")

            width: parent.width - units.gu(4) * 2
            anchors {
                horizontalCenter: parent.horizontalCenter
            }

            onAccepted: {
                var userInput = text

                // Very simple logic, no spaces and contains dots.
                // TODO replace with regexp.
                var isUrlEntered = (userInput.indexOf(" ") === -1 && userInput.indexOf(".") !== -1)

                if (isUrlEntered)
                    googleFeedApi.loadFeed(userInput)
                else googleFeedApi.findFeeds(text)
            }
        }

        ListItem.Header {

            ListItem.ThinDivider { }

            text: i18n.tr("Search results")
        }
    } // Column


    ListView {
        id: searchResults

        width: parent.width
        clip: true
        anchors {
            bottom: parent.bottom
            // bottomMargin: appendFeedTools.height
            top: appendFeedColumn.bottom
        }

        model: searchResultsModel

        delegate: ListItem.Standard {
            text: model.title
            control: CheckBox {
                anchors.verticalCenter: parent.verticalCenter

                onCheckedChanged: {
                    searchResultsModel.setProperty(index, "isSelected", checked)
                    if (checked)
                        selectedCount++
                    else selectedCount--
                }
            }
        }
    }

    ListModel {
        id: searchResultsModel
    }

    Label {
        visible: searchResultsModel.count == 0
        anchors.centerIn: parent
        text: i18n.tr("No feeds")
        fontSize: "large"
    }
}
